#!BPY
# -*- coding: latin-1 -*-

# -------------------------------------------------------------------------- 
#  File: alc_Wiz_Helpers.py 
#  Date: 2011_04_18
# -------------------------------------------------------------------------- 
# ***** BEGIN GPL LICENSE BLOCK ***** 
# 
# Copyright (C) 2011: Rainer Kesselschlaeger aka Tachzusamm
# 
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License 
# as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
# 
# ***** END GPL LICENCE BLOCK ***** 
# -------------------------------------------------------------------------- 


import sys, os

import Blender
from Blender import Window

# DEBUG/DEVEL SWITCHES
wizDebug = False		# Debugging outputs
wizDevel = True		# Special Development treatments (like refreshing included files)

# MODIFIED version of Gow PyPRP AlcScript class
from AlcWIZ.mod_AlcScript import *



# AlcScript-Wizard: Helper functions
# ===================================================

# Get a list of all selected objects in a scene
# ---------------------------------------------
def aw_GetSelObjects_NameList():
    obList = []
    in_emode = Window.EditMode()
    if in_emode: Window.EditMode(0)
    
    scene = Blender.Scene.GetCurrent()
    
    for obj in scene.objects:
        if obj.isSelected():
            #ConvertLocation (obj)
            obList.append(obj.name)
            pass
    
    #if in_emode: Window.EditMode(1)
    #Blender.Redraw()
    return obList
    


# Re-assemble the AlcScript; keep original order of objects and comments and
# append the new objects at the end of the list. Because PyPRP AlcScript class
# does not keep the original order of objects when writing back to Blender,
# we avoid the use of it here and do the export in our own style to keep the
# comments attached to the objects they belong to.
# ----------------------------------------------------------------------------
def aw_ReAssemble_AlcScript(alcList, updateList):
    alcText = ""
    numProc = 0
    
    # First, loop through the initial Script-List to keep the object order
    # --------------------------------------------------------------------
    for obTupel in alcList:
        obName = obTupel[0]
        
        # add leading comment
        alcText += obTupel[1]
        
        # if we made changes to the object dict, add the dict; else add the original text
        if obName in updateList:
            obScript = AlcScript.GetObjectScript(obName)
            if obScript is not None:
                #print obScript
                alcText += obScript
                numProc += 1
            updateList.remove(obName)	# this is done now
        else:
            alcText += obTupel[2]
    
    # Then let's see if we must append newly added objects
    # ----------------------------------------------------
    if len(updateList) > 0:
        if wizDebug:
            Blender.Draw.PupMenu("NOTE:%t|Objects remaining!")
        updateList.sort()
        for obName in updateList:
            obScript = AlcScript.GetObjectScript(obName)
            if obScript is not None:
                # print obScript
                alcText += "\n" + obScript
                numProc += 1
        
    # Finally store in Blender
    # ------------------------
    #blendtext = alcFindBlenderText(AlcScript.ObjectTextFileName)
    blendtext = findBlenderText("AlcScript")
    blendtext.clear()
    blendtext.write(alcText)
    
    return numProc
    


# An ugly function for debugging; works like Print but to (a fixed) textfile
def PrintFile(content):
    if wizDebug:	# (TODO: set to False in final release to avoid unwanted file creation)
        textfile = open("C:\\AS_WIZ_DEBUG.txt", "a")
        print >>textfile, content
        textfile.close()
    
