#!BPY
# -*- coding: latin-1 -*-
"""
Name: 'PyPRP: AlcScript Wizard - Master_GUI'
Blender: 249
Group: 'Wizards'
Tooltip: 'A collection of AlcScript Wizards (GUI).'
"""


# -------------------------------------------------------------------------- 
#  File: alc_Wiz_MasterGUI.py 
#  Date: 2011_04_18
# -------------------------------------------------------------------------- 
# ***** BEGIN GPL LICENSE BLOCK ***** 
# 
# Copyright (C) 2011: Rainer Kesselschlaeger aka Tachzusamm
# 
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License 
# as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
# 
# ***** END GPL LICENCE BLOCK ***** 
# -------------------------------------------------------------------------- 
# GUI design based on / inspired from the Bolt Factory Wizard by Aaron Keith
# See: http://sourceforge.net/projects/boltfactory/
#      http://wiki.blender.org/index.php/Extensions:Py/Scripts/Manual/Misc/Bolt_Factory
# -------------------------------------------------------------------------- 


# IMPORTANT NOTES (as taken from http://www.blender.org/documentation/249PythonDoc/ , Module "Draw"):
# 
# very important: if using your script causes "Error totblock" messages when Blender exits (meaning that memory has been leaked), 
# this may have been caused by an ignored return value from one of the button types. 
# To avoid this, assign created buttons return values to global variables, instead of ignoring them. Examples: 
#       # avoid this, it can cause memory leaks:
#       Draw.Toggle(...)
#       Draw.Number(...)
#       Draw.String(...)
#       # this is correct -- assuming the variables are globals:
#       my_toggle_button = Draw.Toggle(...)
#       my_int_button = Draw.Number(...)
#       my_str_button = Draw.String(...)
# 
# Inside the windowing loop (after Draw.Register() has been executed and before Draw.Exit() is called), 
# don't use the redraw functions from other modules (Blender and Window). 
# The Draw submodule has its own Draw.Redraw() and Draw.Draw() functions that can be used inside the windowing loop. 
#
# ==> Draw Button objects which must have a GLOBAL variable to get their values returned instead of ignoring them:
#     Create, Menu, Toggle, Slider, ColorPicker, Normal, Number, and String (not PushButton?)



import Blender
from Blender import Draw, BGL

# DEBUG/DEVEL SWITCHES
wizDebug = False		# Debugging outputs
wizDevel = True		# Special Development treatments (like refreshing included files)

# IMPORT: VisRegion-Wizard
import AlcWIZ.alc_Wiz_VisRegion
if wizDevel:
    reload(AlcWIZ.alc_Wiz_VisRegion)
from AlcWIZ.alc_Wiz_VisRegion import *

# IMPORT: RenderFlags-Wizard
import AlcWIZ.alc_Wiz_RenderFlags
if wizDevel:
    reload(AlcWIZ.alc_Wiz_RenderFlags)
from AlcWIZ.alc_Wiz_RenderFlags import *


MAX_NAME_LEN = 21	#Names of objects in Blender 2.49 can't be longer


# EVENTS; list like an ENUM
No_Event, \
On_VisRegion_Click, On_RenderFlags_Click, \
On_Change_Click,    On_Select_Click, \
On_Append_Click,    On_Remove_Click, \
On_Apply_Click,     On_Match_Click, \
On_IcyExt_Click,    On_Exit_Click = range(11)


# Definition of Toggle Buttons
# ----------------------------
Main_Menu = {'VisRegions' :  [Draw.Create(1),On_VisRegion_Click,"Wizard for VisRegions"],
             'RenderFlags' : [Draw.Create(0),On_RenderFlags_Click, "Wizard for RenderFlags"]}

ChgSel_Choice = {'CHANGE' : [Draw.Create(1),On_Change_Click,"CHANGE assignment of objects"],
                 'SELECT' : [Draw.Create(0),On_Select_Click,"SELECT assigned objects"]}

AddRem_Choice = {'ADD' : [Draw.Create(1),On_Append_Click,"INCLUDE selected objects"],
                 'SUB' : [Draw.Create(0),On_Remove_Click,"EXCLUDE selected objects"]}

Icicle_Extend = {'EXT' : [Draw.Create(0),On_IcyExt_Click,"Show extended icicle flag options"]}


# Global Storage of values; see IMPORTANT NOTES above
# ---------------------------------------------------
gVisRegion_Name = Draw.Create("<insert name>")

gSelection_Mode = Draw.Create(0)

gRenderLevel_Major = Draw.Create(0)
gRenderLevel_Minor = Draw.Create(0)

gIcicle_Flags = Draw.Create(0)



# Lookup Tables
# -------------
gRL_Major_Table = [None, "opaque", "fb", "defrend", "blend", "late"]
gRL_Minor_Table = [None, "opaque", "fb", "defrend", "blend", "late"]

gIcicle_Table = [None, "nodraw", "noshadowcast",  \
                       "litevtxpreshaded", "litevtxnonpreshaded", \
                       "liteshadow", \
                       "noshadow", "forceshadow", \
                       "disablenormal", "vislos"]

gIcicle_Table_EX = [None, "litematerial", "nodraw", "noshadowcast", "facessortable", \
                          "volatile", "waterheight", "runtimelight", "reversesort", \
                          "haspermalights", "haspermaprojs", "litevtxpreshaded", "litevtxnonpreshaded", \
                          "liteprojection", "liteshadowerase", "liteshadow", "mathasspecular", \
                          "projasvtx", "skipprojection", "noshadow", "forceshadow", \
                          "disablenormal", "character", "partialsort", "vislos"]



##################################################################################################

def Generate_Menu_String(table):
    menu=""
    for i in xrange(1, len(table)):
        menu += "| " + table[i] + " %x" + str(i)
    return menu

##################################################################################################



###############################################################################
#                              EVENT HANDLING
###############################################################################

def event(evt, val):    # the function to handle input events

    if evt == Draw.ESCKEY:
        Draw.Exit()                 # exit when user presses ESC
        return


def button_event(evt):  # the function to handle Draw Button events

    if evt == On_Exit_Click:
        Draw.Exit()                 # exit when user clicks EXIT
        return
    
    
    if evt == On_Match_Click:
        selMode = gSelection_Mode.val
        
        if Main_Menu['VisRegions'][0].val:
            vrName = gVisRegion_Name.val
            
            # EXEC
            retStr = aw_Select_Objects_In_VisRegion(selMode, vrName)
            msg = "RESULT:%t|"+retStr
            Blender.Draw.PupMenu(msg)
        
        if Main_Menu['RenderFlags'][0].val:
            MajorName = gRL_Major_Table[gRenderLevel_Major.val]
            MinorName = gRL_Minor_Table[gRenderLevel_Minor.val]
            
            if Icicle_Extend['EXT'][0].val == 0:	# NORMAL
                IcicleName = gIcicle_Table[gIcicle_Flags.val]
            else:									# EXTENDED
                IcicleName = gIcicle_Table_EX[gIcicle_Flags.val]
            
            # EXEC
            retStr = aw_Select_Objects_By_RenderFlags(selMode, MajorName, MinorName, IcicleName)
            msg = "RESULT:%t|"+retStr
            Blender.Draw.PupMenu(msg)
        
        Draw.Redraw(1)
    
    
    if evt == On_Apply_Click:
        addMode = AddRem_Choice['ADD'][0].val
        if wizDebug:
            if addMode == 1:  Blender.Draw.PupMenu("AddMode:%t|ON")
            else:             Blender.Draw.PupMenu("AddMode:%t|OFF")
        
        if Main_Menu['VisRegions'][0].val:
            vrName = gVisRegion_Name.val
            
            if wizDebug:
                msg = "VisRegion:%t|" + vrName
                Blender.Draw.PupMenu(msg)
            
            # EXECUTE
            retStr = asWiz_Add_SelObjects_VisRegion(addMode, vrName)
            msg = "RESULT:%t|"+retStr
            Blender.Draw.PupMenu(msg)
        
        if Main_Menu['RenderFlags'][0].val:
            MajorName = gRL_Major_Table[gRenderLevel_Major.val]
            MinorName = gRL_Minor_Table[gRenderLevel_Minor.val]
            
            if Icicle_Extend['EXT'][0].val == 0:	# NORMAL
                IcicleName = gIcicle_Table[gIcicle_Flags.val]
            else:									# EXTENDED
                IcicleName = gIcicle_Table_EX[gIcicle_Flags.val]
            
            if wizDebug:
                if MajorName is not None:	Blender.Draw.PupMenu("MajorName:%t|" + MajorName)
                if MinorName is not None:	Blender.Draw.PupMenu("MinorName:%t|" + MinorName)
                if IcicleName is not None:	Blender.Draw.PupMenu("IcicleName:%t|" + IcicleName)
            
            # EXECUTE
            retStr = asWiz_Set_SelObjects_RenderFlags(addMode, MajorName, MinorName, IcicleName)
            msg = "RESULT:%t|"+retStr
            Blender.Draw.PupMenu(msg)
        
        Draw.Redraw(1)
    
    
    elif evt == On_IcyExt_Click:
    #    if Icicle_Extend['EXT'][1]!=evt:
    #        Icicle_Extend['EXT'][0].val=0
    #    else:
    #        Icicle_Extend['EXT'][0].val=1
        gIcicle_Flags.val = 0	# Reset selection to None
        if Icicle_Extend['EXT'][0].val == 1:
            Blender.Draw.PupBlock("WARNING:", ["This is EXPERIMENTAL !", \
                                               "It expands the available icicle", \
                                               "flag options to the full range.", \
                                               "Some of these ext.flags are", \
                                               "set by PyPRP, use with care !"])
        Draw.Redraw(1)
    
    
    elif (evt in [On_VisRegion_Click, On_RenderFlags_Click]):
        for k in Main_Menu.iterkeys():
            if Main_Menu[k][1]!=evt:
                Main_Menu[k][0].val=0
            else:
                Main_Menu[k][0].val=1
        Draw.Redraw(1)
    
    elif (evt in [On_Change_Click, On_Select_Click]):
        for k in ChgSel_Choice.iterkeys():
            if ChgSel_Choice[k][1]!=evt:
                ChgSel_Choice[k][0].val=0
            else:
                ChgSel_Choice[k][0].val=1    
        Draw.Redraw(1)
    
    elif (evt in [On_Append_Click, On_Remove_Click]):
        for k in AddRem_Choice.iterkeys():
            if AddRem_Choice[k][1]!=evt:
                AddRem_Choice[k][0].val=0
            else:
                AddRem_Choice[k][0].val=1
        Draw.Redraw(1)
    
    

###############################################################################
###############################################################################
#                              DRAWING THE GUI
###############################################################################
###############################################################################

def Draw_Border(X1,Y1,X2,Y2): # X1,Y1 = Top Left X2,Y2 = Bottom Right
    INDENT = 3
    
    BGL.glColor3f(1.0,1.0,1.0)
    BGL.glBegin(BGL.GL_LINES)                
    BGL.glVertex2i(X1+INDENT,Y1-INDENT)     #top line
    BGL.glVertex2i(X2-INDENT,Y1-INDENT)
    
    BGL.glVertex2i(X1+INDENT,Y1-INDENT)     #left line
    BGL.glVertex2i(X1+INDENT,Y2+INDENT)
    BGL.glEnd()

    BGL.glColor3f(0.5,0.5,0.5)
    BGL.glBegin(BGL.GL_LINES)                
    BGL.glVertex2i(X2-INDENT,Y1-INDENT)     #Right line
    BGL.glVertex2i(X2-INDENT,Y2+INDENT)
    
    BGL.glVertex2i(X1+INDENT,Y2+INDENT)     #bottom line
    BGL.glVertex2i(X2-INDENT,Y2+INDENT)
    BGL.glEnd()
    


# General Action Display
# ================================================================================
def Display_Action (txtList, Y_POS):
    Draw.Label(txtList[0], 15,Y_POS,    220, 12)
    Draw.Label(txtList[1], 15,Y_POS-18, 220, 12)
    


# General Buttons
# ================================================================================
def Display_Apply_Button (Y_POS):
    Draw.PushButton("APPLY", On_Apply_Click, 40, Y_POS, 110, 18, "Apply changes (updates AlcScript)")
    

def Display_Match_Button (Y_POS):
    Draw.PushButton("Find Match", On_Match_Click, 40, Y_POS, 110, 18, "Find and select matching objects")
    

def Display_Exit_Button (Y_POS):
    Draw.PushButton("EXIT", On_Exit_Click, 180, Y_POS, 50, 18, "Quit (close wizard)")
    


# General Choices
# ================================================================================
def Display_Change_Select_Menu(Y_POS):
    
    Buttons = ChgSel_Choice
    
    BUT_WIDTH = 80
    BUT_HEIGHT = 18
    Button_X = 30
    
    key = 'CHANGE'
    Buttons[key][0]= Draw.Toggle(key, Buttons[key][1], Button_X,Y_POS, BUT_WIDTH,BUT_HEIGHT, Buttons[key][0].val, Buttons[key][2])
    Button_X += BUT_WIDTH + 30
    key = 'SELECT'
    Buttons[key][0]= Draw.Toggle(key, Buttons[key][1], Button_X,Y_POS, BUT_WIDTH,BUT_HEIGHT, Buttons[key][0].val, Buttons[key][2])
    

def Display_Insert_Remove_Menu(Y_POS):
    
    Buttons = AddRem_Choice
    
    BUT_WIDTH = 40
    BUT_HEIGHT = 18
    Button_X = 30
    
    key = 'ADD'
    Buttons[key][0]= Draw.Toggle(key, Buttons[key][1], Button_X,Y_POS, BUT_WIDTH,BUT_HEIGHT, Buttons[key][0].val, Buttons[key][2])
    Button_X += BUT_WIDTH + 0
    key = 'SUB'
    Buttons[key][0]= Draw.Toggle(key, Buttons[key][1], Button_X,Y_POS, BUT_WIDTH,BUT_HEIGHT, Buttons[key][0].val, Buttons[key][2])
    


# Selection Mode
# ================================================================================
def Display_Selection_Menu(Y_POS):
    
    global gSelection_Mode
    SelName = "Selection Mode: %t| New selection %x0| Add to selected %x1| Subset of selection %x2| Subtract from select. %x3"
    Draw.Label("Selection Mode:", 70, Y_POS, 150, 12)
    gSelection_Mode = Draw.Menu(SelName, No_Event, 70,  Y_POS-20, 150,18, gSelection_Mode.val, "defines how the next selection is done")
    


# VisRegion-Functions
# ================================================================================
def Display_VisRegion_Value(Y_POS):
    
    global gVisRegion_Name
    gVisRegion_Name = Draw.String("VisRegion: ", 0, 15, Y_POS, 220,18, gVisRegion_Name.val, MAX_NAME_LEN, "Enter or paste the name of your VisRegion")
    


# RenderFlags-Functions
# ================================================================================
def Display_RenderFlags_Values(Y_POS):
    
    global gRenderLevel_Major
    global gRenderLevel_Minor
    
    MajorName = "Major: %t| %x0" + Generate_Menu_String(gRL_Major_Table)
    MinorName = "Minor: %t| %x0" + Generate_Menu_String(gRL_Minor_Table)
    
    Draw.Label("Major:", 25, Y_POS, 80, 12)
    Draw.Label("Minor:", 125,Y_POS, 80, 12)    
    
    gRenderLevel_Major = Draw.Menu(MajorName, No_Event, 25,  Y_POS-20, 80,18, gRenderLevel_Major.val, "visual.renderlevel.major")
    gRenderLevel_Minor = Draw.Menu(MinorName, No_Event, 125, Y_POS-20, 80,18, gRenderLevel_Minor.val, "visual.renderlevel.minor")
    
	

def Display_Icicle_Values(Y_POS):
    
    global gIcicle_Flags
    if Icicle_Extend['EXT'][0].val == 0:	# NORMAL
        IcicleName = "Icicle: %t| %x0" + Generate_Menu_String(gIcicle_Table)
    else:									# EXTENDED
        IcicleName = "Icicle: %t| %x0" + Generate_Menu_String(gIcicle_Table_EX)
    #print IcicleName
    Draw.Label("Icicle:", 25, Y_POS, 150, 12)
    gIcicle_Flags = Draw.Menu(IcicleName, No_Event, 25,  Y_POS-20, 150,18, gIcicle_Flags.val, "visual.icicle")
    
    Buttons = Icicle_Extend
    BUT_WIDTH = 30
    BUT_HEIGHT = 18
    Button_X = 190
    key = 'EXT'
    Buttons[key][0]= Draw.Toggle(key, Buttons[key][1], Button_X,Y_POS-20, BUT_WIDTH,BUT_HEIGHT, Buttons[key][0].val, Buttons[key][2])
    
	

# Main Wizard Area
# ================================================================================
def Display_Wizard_Area(WIZ_HEIGHT, WIZ_WIDTH):
    
    Title = " AlcScript GUI V0.20"
    Buttons = Main_Menu
    
    MAIN_HEIGHT = 55
    
    X1 = 0
    Y1 = WIZ_HEIGHT
    X2 = WIZ_WIDTH
    Y2 = WIZ_HEIGHT - MAIN_HEIGHT
    
    BUT_WIDTH = 100
    BUT_HEIGHT = 18
    BUT_INDENT = 18
    BUT_GAP = 10
    
    # Frames
    BGL.glColor3f(0.75, 0.75, 0.75)
    BGL.glRecti(X1,Y1,X2,0)
    Draw_Border(X1,Y1,X2,Y2)	# Main_Menu
    Draw_Border(X1,Y2,X2,0)		# Work Menu
    
    # Title
    BGL.glColor3f(0.0,0.0,1.0)
    BGL.glRasterPos2d(X1+BUT_INDENT+40, Y1 + 5)
    Draw.Text(Title)
    
    Button_X = X1 + BUT_INDENT
    Button_Y = Y1 - BUT_HEIGHT - 16
    
    key = 'VisRegions'
    Buttons[key][0]= Draw.Toggle(key, Buttons[key][1], Button_X,Button_Y, BUT_WIDTH,BUT_HEIGHT, Buttons[key][0].val, Buttons[key][2])
    Button_X += BUT_WIDTH + BUT_GAP
    key = 'RenderFlags'
    Buttons[key][0]= Draw.Toggle(key, Buttons[key][1], Button_X,Button_Y, BUT_WIDTH,BUT_HEIGHT, Buttons[key][0].val, Buttons[key][2])
    
    return WIZ_HEIGHT - MAIN_HEIGHT
    


# Displaying the TABs
# ================================================================================
def Display_VisRegion_Tab(WORK_HEIGHT):
       
    Display_VisRegion_Value(WORK_HEIGHT - 35)
    Display_Change_Select_Menu(WORK_HEIGHT - 70)
    
    if ChgSel_Choice['CHANGE'][0].val:
        Display_Insert_Remove_Menu(WORK_HEIGHT - 110)
        
        if AddRem_Choice['ADD'][0].val:
            Display_Action (["Action: INCLUDE selected objects", " in VisRegion"], WORK_HEIGHT - 140)
        else:
            Display_Action (["Action: EXCLUDE selected objects", " from VisRegion"], WORK_HEIGHT - 140)
        
        Display_Apply_Button(20)
    
    if ChgSel_Choice['SELECT'][0].val:
        Display_Selection_Menu(WORK_HEIGHT - 100)
        
        Display_Action (["Action: SELECT all objects", " currently assigned to VisRegion"], WORK_HEIGHT - 150)
        
        Display_Match_Button(20)
    


def Display_RenderFlags_Tab(WORK_HEIGHT):
    
    Display_RenderFlags_Values(WORK_HEIGHT - 25)
    Display_Icicle_Values(WORK_HEIGHT - 65)
    Display_Change_Select_Menu(WORK_HEIGHT - 120)
    
    if ChgSel_Choice['CHANGE'][0].val:
        Display_Insert_Remove_Menu(WORK_HEIGHT - 155)
        
        if AddRem_Choice['ADD'][0].val:
            Display_Action (["Action: SET flags on selected objects", ""], WORK_HEIGHT - 180)
        else:
            Display_Action (["Action: CLEAR flags of select. objects", ""], WORK_HEIGHT - 180)
        
        Display_Apply_Button(20)
    
    if ChgSel_Choice['SELECT'][0].val:
        Display_Selection_Menu(WORK_HEIGHT - 140)
        
        Display_Action (["Action: SELECT all objects", " matching the chosen flags"], WORK_HEIGHT - 180)
        
        Display_Match_Button(20)
    
    pass
    


##########################################################################################

def gui():              # the function to draw the screen
    
    WIZ_WIDTH = 250
    WIZ_HEIGHT = 300

    BGL.glClearColor(0.6, 0.6, 0.6, 1.0)
    BGL.glClear(BGL.GL_COLOR_BUFFER_BIT)
    
    WORK_HEIGHT = Display_Wizard_Area(WIZ_HEIGHT, WIZ_WIDTH)
    
    if Main_Menu['VisRegions'][0].val:
        Display_VisRegion_Tab(WORK_HEIGHT)
    
    elif Main_Menu['RenderFlags'][0].val:
        Display_RenderFlags_Tab(WORK_HEIGHT)
    
    Display_Exit_Button (20)
    
Draw.Register(gui, event, button_event)  # registering the 3 callbacks
