#!BPY
# -*- coding: latin-1 -*-

# This script can be used from the AlcScript-Wizard-MasterGUI-Menu


# -------------------------------------------------------------------------- 
#  File: alc_Wiz_RenderFlags.py 
#  Date: 2011_04_18
# -------------------------------------------------------------------------- 
# ***** BEGIN GPL LICENSE BLOCK ***** 
# 
# Copyright (C) 2011: Rainer Kesselschlaeger aka Tachzusamm
# 
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License 
# as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
# 
# ***** END GPL LICENCE BLOCK ***** 
# -------------------------------------------------------------------------- 


import sys, os
import yaml

import Blender
from Blender import Window, Mesh

# DEBUG/DEVEL SWITCHES
wizDebug = False		# Debugging outputs
wizDevel = True		# Special Development treatments (like refreshing included files)

# MODIFIED version of Gow PyPRP AlcScript class
from AlcWIZ.mod_AlcScript import *

import AlcWIZ.alc_Wiz_Helpers
if wizDevel:
    reload(AlcWIZ.alc_Wiz_Helpers)
from AlcWIZ.alc_Wiz_Helpers import *

import AlcWIZ.alc_PreParse
if wizDevel:
    reload(AlcWIZ.alc_PreParse)
from AlcWIZ.alc_PreParse import *

MAX_NAME_LEN = 21	#Names of objects in Blender 2.49 can't be longer



# Find + select all objects having the given flags set
# ----------------------------------------------------
def aw_Select_Objects_By_RenderFlags(selMode, MajorName, MinorName, IcicleName):
    
    in_emode = Window.EditMode()
    if in_emode: Window.EditMode(0)
    
    # If no flags set, quit w/Error
    # -----------------------------
    if MajorName is None and MinorName is None and IcicleName is None:
        #Blender.Draw.PupBlock("ERROR:", ["No flags defined!"])
        return "No flags defined!"
    
    # Determine how many flags must match (AND-Style logic)
    # -----------------------------------------------------
    numFlagsNeed = 0
    if MajorName is not None:	numFlagsNeed += 1
    if MinorName is not None:	numFlagsNeed += 1
    if IcicleName is not None:	numFlagsNeed += 1
    
    
    scene = Blender.Scene.GetCurrent()
    
    # Selection Modes: New selection:0, Add to selected:1, Subset of selection:2, Subtract from select.:3
    if selMode == 0:
        # unselect all
        scene.objects.selected = []
    
    AlcScript.LoadFromBlender()
    alcDict = AlcScript.objects.GetRootScript()
    
    if wizDebug:
        PrintFile(alcDict)
        PrintFile("\n\n")
    
    obList = []
    for obName in alcDict.keys():
        numFlagsFound = 0
        if wizDebug:
            PrintFile(obName)
        
        # Now search for our flags
        obDict = alcDict[obName]
        
        # skip non-dictionaries
        if not type(obDict) == dict: continue
        #
        if MajorName is not None:
            FlagList = FindInDict(obDict, 'visual.renderlevel.major')
            if FlagList is not None:
                if MajorName in FlagList:
                    numFlagsFound += 1
        #
        if MinorName is not None:
            FlagList = FindInDict(obDict, 'visual.renderlevel.minor')
            if FlagList is not None:
                if MinorName in FlagList:
                    numFlagsFound += 1
        #
        if IcicleName is not None:
            FlagList = FindInDict(obDict, 'visual.icicle')
            if FlagList is not None:
                if IcicleName in FlagList:
                    numFlagsFound += 1
        
        if numFlagsFound == numFlagsNeed:
            obList.append(obName)
        
    if wizDebug:
        PrintFile(obList)
    
    if len(obList) > 0:
        if selMode == 0:	# New selection
            obs = [ob for ob in scene.objects if ob.name in obList]
        elif selMode == 1:	# Add to selected
            obs = [ob for ob in scene.objects if (ob.name in obList or ob in scene.objects.selected)]
        elif selMode == 2:	# Subset of selection
            obs = [ob for ob in scene.objects if (ob.name in obList and ob in scene.objects.selected)]
        elif selMode == 3:	# Subtract from select.
            obs = [ob for ob in scene.objects if (not ob.name in obList and ob in scene.objects.selected)]
        scene.objects.selected = obs
    else:
        if selMode == 2:	# Subset of selection (in this case, nothing remains)
            scene.objects.selected = []
    
    #if in_emode: Window.EditMode(1)
   # Blender.Redraw()
    
    numSel = len(scene.objects.selected)
    if numSel == 0:
        msg = "No objects selected."
    elif numSel == 1:
        msg = "One object selected."
    else:
        msg = "%d objects selected." % numSel
    return msg
    


# AlcScript-Wizard: Set given flags on all selected objects (or clear them)
# ==========================================================================
def asWiz_Set_SelObjects_RenderFlags(addMode, MajorName, MinorName, IcicleName):
    
    # 1. Get a list of selected object names 
    #    If no objects selected, quit w/error
    # =======================================
    obNameList = aw_GetSelObjects_NameList()
    if obNameList is None or len(obNameList) == 0:
        #Blender.Draw.PupBlock("ERROR:", ["No objects selected!"])
       # Blender.Draw.PupMenu("ERROR:%t|No objects selected!")
        return "No objects selected!"
    
    # If no flags set, quit as well
    # -----------------------------
    if MajorName is None and MinorName is None and IcicleName is None:
        Blender.Draw.PupBlock("ERROR:", ["No flags defined!"])
        return "No flags defined!"
    
    
    # Start hourglass (could take longer)
    Window.WaitCursor(1)
    
    
    # 2. Read the current AlcScript, parse in two different ways:
    #    a) Read line by line to get the order of objects, and to store comments;
    #       additionally, store the original text content per object for later use.
    #    b) Parse the script with PyYaml to get a dict of existing objects created
    #       (just using a modified PyPRP AlcScript class here).
    # ============================================================================
    alcList = aw_PreParse_AlcScript()
   # print alcList
    
    AlcScript.LoadFromBlender()
    
    
    # 3. In the dict, update existing objects; set or clear flags on those in list
    #    Or if object is not found in dict, add the object with flag properties
    # ==============================================================================
    updateList = []
    if addMode == 1:	# SET object flags
        for obName in obNameList:
            needUpdate = False
            obDict = AlcScript.objects.FindStrict(obName)
            if obDict is not None:
                #
                if MajorName is not None:
                    FlagList = FindInDict(obDict, 'visual.renderlevel.major')
                    #if FlagList is not None and type(FlagList) == list:		# (could FindInDict return direct values instead of None or list?)
                    if FlagList is not None:
                        if not MajorName in FlagList:
                            FlagList.append(MajorName)
                            StoreInDict(obDict, "visual.renderlevel.major", FlagList)
                            needUpdate = True
                    # no flags yet available? then create a new list
                    else:
                        FlagList = [MajorName]
                        StoreInDict(obDict, "visual.renderlevel.major", FlagList)
                        needUpdate = True
                #
                if MinorName is not None:
                    FlagList = FindInDict(obDict, 'visual.renderlevel.minor')
                    #if FlagList is not None and type(FlagList) == list:
                    if FlagList is not None:
                        if not MinorName in FlagList:
                            FlagList.append(MinorName)
                            StoreInDict(obDict, "visual.renderlevel.minor", FlagList)
                            needUpdate = True
                    else:
                        FlagList = [MinorName]
                        StoreInDict(obDict, "visual.renderlevel.minor", FlagList)
                        needUpdate = True
                        
                #
                if IcicleName is not None:
                    FlagList = FindInDict(obDict, 'visual.icicle')
                    #if FlagList is not None and type(FlagList) == list:
                    if FlagList is not None:
                        if not IcicleName in FlagList:
                            FlagList.append(IcicleName)
                            StoreInDict(obDict, "visual.icicle", FlagList)
                            needUpdate = True
                    else:
                        FlagList = [IcicleName]
                        StoreInDict(obDict, "visual.icicle", FlagList)
                        needUpdate = True
                        
                
                # Mark this object as "updated", so the dict will be used on AlcScript-Assembly
                if needUpdate:
                    updateList.append(obName)
            
            # create a new AlcScript entry for objects not yet listed
            else:
               obDict = AlcScript.objects.FindOrCreate(obName)
               #
               if MajorName is not None:
                   FlagList = [MajorName]
                   StoreInDict(obDict, "visual.renderlevel.major", FlagList)
               if MinorName is not None:
                   FlagList = [MinorName]
                   StoreInDict(obDict, "visual.renderlevel.minor", FlagList)
               if IcicleName is not None:
                   FlagList = [IcicleName]
                   StoreInDict(obDict, "visual.icicle", FlagList)
               updateList.append(obName)
    
    
    else:	# CLEAR object flags
        for obName in obNameList:
            needUpdate = False
            obDict = AlcScript.objects.FindStrict(obName)
            # skip if object not in AlcScript
            if obDict is None:
                continue
            #
            if MajorName is not None:
                FlagList = FindInDict(obDict, 'visual.renderlevel.major')
                if FlagList is not None:
                    if MajorName in FlagList:
                        FlagList.remove(MajorName)
                        StoreInDict(obDict, "visual.renderlevel.major", FlagList)
                        needUpdate = True
                        
                        if wizDebug:
                            PrintFile(obDict)
                            PrintFile("\n\n")
            #
            if MinorName is not None:
                FlagList = FindInDict(obDict, 'visual.renderlevel.minor')
                if FlagList is not None:
                    if MinorName in FlagList:
                        FlagList.remove(MinorName)
                        StoreInDict(obDict, "visual.renderlevel.minor", FlagList)
                        needUpdate = True
            #
            if IcicleName is not None:
                FlagList = FindInDict(obDict, 'visual.icicle')
                if FlagList is not None:
                    if IcicleName in FlagList:
                        FlagList.remove(IcicleName)
                        StoreInDict(obDict, "visual.icicle", FlagList)
                        needUpdate = True
            
            if needUpdate:
                CleanupDict(obDict, delMode = "remBoth")
                if wizDebug:
                    PrintFile(obDict)
                    PrintFile("(cleaned)\n\n\n")
                updateList.append(obName)
            
    
    # exit if nothing to do
    if len(updateList) == 0:
        Window.WaitCursor(0)
        return "No objects to process!"
    
    
    # 4. Finally re-assemble the AlcScript; keep original order of objects and comments
    #    and append the new objects at the end of the list. Because PyPRP AlcScript 
    #    class does not keep the original order of objects when writing back to Blender,
    #    we avoid the use of it here and do the export in our own style to keep the
    #    comments attached to the objects they belong to.
    # =================================================================================
    numProc = aw_ReAssemble_AlcScript(alcList, updateList)
    
    Window.WaitCursor(0)	# Job done
    
    if numProc == 0:
        msg = "No objects processed."
    elif numProc == 1:
        msg = "One object processed."
    else:
        msg = "%d objects processed." % numProc
    return msg
    

