#!BPY
# -*- coding: latin-1 -*-
"""
Name: 'PyPRP: AlcScript Wizard - VisRegion'
Blender: 249
Group: 'Object'
Tooltip: 'Add selected objects to a VisRegion, or remove them from a region.'
"""
#Group: 'Wizards'


# This script can be used from the object menu of from the AlcScript-Wizard-MasterGUI-Menu


# -------------------------------------------------------------------------- 
#  File: alc_Wiz_VisRegion.py 
#  Date: 2011_04_18
# -------------------------------------------------------------------------- 
# ***** BEGIN GPL LICENSE BLOCK ***** 
# 
# Copyright (C) 2011: Rainer Kesselschlaeger aka Tachzusamm
# 
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License 
# as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version. 
# 
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software Foundation, 
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
# 
# ***** END GPL LICENCE BLOCK ***** 
# -------------------------------------------------------------------------- 


import sys, os
import yaml

import Blender
from Blender import Window, Mesh

# DEBUG/DEVEL SWITCHES
wizDebug = False		# Debugging outputs
wizDevel = True		# Special Development treatments (like refreshing included files)

# MODIFIED version of Gow PyPRP AlcScript class
from AlcWIZ.mod_AlcScript import *

import AlcWIZ.alc_Wiz_Helpers
if wizDevel:
    reload(AlcWIZ.alc_Wiz_Helpers)
from AlcWIZ.alc_Wiz_Helpers import *

import AlcWIZ.alc_PreParse
if wizDevel:
    reload(AlcWIZ.alc_PreParse)
from AlcWIZ.alc_PreParse import *

MAX_NAME_LEN = 21	#Names of objects in Blender 2.49 can't be longer



# Find + select all objects belonging to a VisRegion
# --------------------------------------------------
def aw_Select_Objects_In_VisRegion(selMode, vrName):
    
    in_emode = Window.EditMode()
    if in_emode: Window.EditMode(0)
    
    # If no VisReg set, quit w/Error
    # ------------------------------
    if vrName is None or vrName.strip() == "" or vrName.strip() == "<insert name>":
        #Blender.Draw.PupBlock("ERROR:", ["No VisRegion defined!"])
        return "No VisRegion defined!"
    
    
    scene = Blender.Scene.GetCurrent()
    
    # Selection Modes: New selection:0, Add to selected:1, Subset of selection:2, Subtract from select.:3
    if selMode == 0:
        # unselect all
        scene.objects.selected = []
    
    AlcScript.LoadFromBlender()
    alcDict = AlcScript.objects.GetRootScript()
    
    if wizDebug:
        PrintFile(alcDict)
        PrintFile("\n\n")
    
    obList = []
    for obName in alcDict.keys():
        if wizDebug:
            PrintFile(obName)
        
        obDict = alcDict[obName]
        
        # skip non-dictionaries
        if not type(obDict) == dict: continue
        
        VisRegList = FindInDict(obDict, 'visual.visregions')
        if VisRegList is not None:
            if vrName in VisRegList:
                obList.append(obName)
        
    if wizDebug:
        PrintFile(obList)
    
    if len(obList) > 0:
        if selMode == 0:	# New selection
            obs = [ob for ob in scene.objects if ob.name in obList]
        elif selMode == 1:	# Add to selected
            obs = [ob for ob in scene.objects if (ob.name in obList or ob in scene.objects.selected)]
        elif selMode == 2:	# Subset of selection
            obs = [ob for ob in scene.objects if (ob.name in obList and ob in scene.objects.selected)]
        elif selMode == 3:	# Subtract from select.
            obs = [ob for ob in scene.objects if (not ob.name in obList and ob in scene.objects.selected)]
        scene.objects.selected = obs
    else:
        if selMode == 2:	# Subset of selection (in this case, nothing remains)
            scene.objects.selected = []
    
    #if in_emode: Window.EditMode(1)
   # Blender.Redraw()
    
    numSel = len(scene.objects.selected)
    if numSel == 0:
        msg = "No objects selected."
    elif numSel == 1:
        msg = "One object selected."
    else:
        msg = "%d objects selected." % numSel
    return msg
    


# AlcScript-Wizard: Add all selected objects to a VisRegion (or remove them)
# ==========================================================================
def asWiz_Add_SelObjects_VisRegion(addMode, vrName):
    
    # 1. Get a list of selected object names 
    #    If no objects selected, quit w/error
    # =======================================
    obNameList = aw_GetSelObjects_NameList()
    if obNameList is None or len(obNameList) == 0:
        #Blender.Draw.PupBlock("ERROR:", ["No objects selected!"])
       # Blender.Draw.PupMenu("ERROR:%t|No objects selected!")
        return "No objects selected!"
    
    # If no VisReg set, quit w/Error
    # ------------------------------
    if vrName is None or vrName.strip() == "" or vrName.strip() == "<insert name>":
        #Blender.Draw.PupBlock("ERROR:", ["No VisRegion defined!"])
        return "No VisRegion defined!"
    
    # check if the chosen object exists, and if it's really a region (TODO)
    # Well, we better do this check only on AddMode; maybe some old stuff
    # has to be removed, regardless if the attached region still exists.
    # ---------------------------------------------------------------------
    if addMode == 1:
        scene = Blender.Scene.GetCurrent()
        obs = [ob for ob in scene.objects if ob.name == vrName]
        if len(obs) == 0:
            return "Region '%s' is not a valid object!" % vrName
    
    
    # Start hourglass (could take longer)
    Window.WaitCursor(1)
    
    
    # 2. Read the current AlcScript, parse in two different ways:
    #    a) Read line by line to get the order of objects, and to store comments;
    #       additionally, store the original text content per object for later use.
    #    b) Parse the script with PyYaml to get a dict of existing objects created
    #       (just using a modified PyPRP AlcScript class here).
    # ============================================================================
    alcList = aw_PreParse_AlcScript()
   # print alcList
    
    AlcScript.LoadFromBlender()
    
    
    # 3. In the dict, update existing objects; append the VisRegion to those in list
    #    Or if object is not found in dict, add the object with VisRegion properties
    # ==============================================================================
    updateList = []
    if addMode == 1:
        for obName in obNameList:
            obDict = AlcScript.objects.FindStrict(obName)
            if obDict is not None:
                VisRegList = FindInDict(obDict, 'visual.visregions')
                if VisRegList is not None:
                    # if the object is already attached to this region, skip it
                    if vrName in VisRegList:
                        continue
                    # else append this region to the list
                    VisRegList.append(vrName)
                    
                    if wizDebug:
                        print VisRegList
                        msg = "vrName:%t|" + vrName
                        Blender.Draw.PupMenu(msg)
                
                # no VisRegions yet available? then create a new list
                else:
                    VisRegList = [vrName]
                
                # update the object's dict
                StoreInDict(obDict, "visual.visregions", VisRegList)
                
                # And Mark this object as "updated", so the dict will be used on AlcScript-Assembly
                updateList.append(obName)
            
            # create a new AlcScript entry for objects not yet listed
            else:
               obDict = AlcScript.objects.FindOrCreate(obName)
               VisRegList = [vrName]
               StoreInDict(obDict, "visual.visregions", VisRegList)
               updateList.append(obName)
    
    else:	# REMOVE objects from VisRegion
        for obName in obNameList:
            obDict = AlcScript.objects.FindStrict(obName)
            # skip if object not in AlcScript or VisRegion not attached
            if obDict is None:
                continue
            VisRegList = FindInDict(obDict, 'visual.visregions')
            if VisRegList is None:
                continue
            if not vrName in VisRegList:
                continue
            
            # now remove the region from the list
            VisRegList.remove(vrName)
            StoreInDict(obDict, "visual.visregions", VisRegList)
            CleanupDict(obDict, delMode = "remBoth")
            updateList.append(obName)
            
    
    # exit if nothing to do
    if len(updateList) == 0:
        Window.WaitCursor(0)
        return "No objects to process!"
    
    
    # 4. Finally re-assemble the AlcScript; keep original order of objects and comments
    #    and append the new objects at the end of the list. Because PyPRP AlcScript 
    #    class does not keep the original order of objects when writing back to Blender,
    #    we avoid the use of it here and do the export in our own style to keep the
    #    comments attached to the objects they belong to.
    # =================================================================================
    numProc = aw_ReAssemble_AlcScript(alcList, updateList)
    
    Window.WaitCursor(0)	# Job done
    
    if numProc == 0:
        msg = "No objects processed."
    elif numProc == 1:
        msg = "One object processed."
    else:
        msg = "%d objects processed." % numProc
    return msg
    


# This is for making this Wizard accessible via object menu
# ---------------------------------------------------------
def askAction():
    visReg_Name = Blender.Draw.Create("<insert name>")
    userAddMode = Blender.Draw.Create(0)
    block = []
    block.append("Apply selected objects to:")
    block.append(("VisRegion: ", visReg_Name, 0, MAX_NAME_LEN, "insert name of VisRegion"))
    block.append("Check to ADD, else REMOVE")
    block.append(("ADD", userAddMode, "Add selected objects to region (checked/dark) or remove them (unchecked/bright)"))
    retval = Blender.Draw.PupBlock("AlcScript Wizard", block)
    
    # retval 0 means abort, 1 = continue
    if retval != 1:
        return	#exit
    
    vrName = visReg_Name.val
    addMode = userAddMode.val
    
    if vrName is None or vrName.strip() == "" or vrName.strip() == "<insert name>":
        Blender.Draw.PupBlock("ERROR:", ["No VisRegion defined!"])
        return	#exit
    
    asWiz_Add_SelObjects_VisRegion(addMode, vrName)
    


# -----------------------------------------------------------------------------
def main():
    askAction()

if __name__ == '__main__':
    main()
